#!/bin/sh
#MiniUPnP project
#author: Thomas Bernard
#website: http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
#control script for Routertech port mstombs Dec 07/Feb 08
#called from binary upnpd which is started by core logic
ME=miniupnpcmd.sh
#full path
FP=/usr/sbin/$ME
MUD=MINIUPNPD
IPT=/sbin/iptables
IPTn="$IPT -t nat"
UPS=/etc/ppp/ip-up
DNS=/etc/ppp/ip-down
LGR=/usr/local/bin/logger
CNF=/var/tmp/miniupnpd.conf

LG(){
	echo "$ME: $@"|$LGR -s
}
EX(){
	LG "Error - $1, exiting"
	exit 1
}

[ -z "$3" ]&& EX "invalid params: $@"

WIF="$2"
WIP="$3"

DIVERT(){
#check if entry already there
	ln=$($IPTn -nL PREROUTING --line-numbers|grep $MUD)
	n=${ln%% *}
	if [ "$n" -gt "0" ];then #replace
		$IPTn -R PREROUTING $n -d $WIP -i $WIF -j $MUD
	else #new
#check for DMZ
		ln=$($IPTn -nL PREROUTING --line-numbers|grep all|grep DNAT)
		n=${ln%% *}
		if [ "$n" -gt "0" ];then #insert so goes above DMZ
			$IPTn -I PREROUTING $n -d $WIP -i $WIF -j $MUD
		else
			$IPTn -A PREROUTING -d $WIP -i $WIF -j $MUD
		fi
	fi
}

UNDIVERT(){
#try this - doesn't work if WIP changed
	$IPTn -D PREROUTING -d $WIP -i $WIF -j $MUD
#clean up any extra entries
	while $IPTn -nL PREROUTING|grep $MUD
	do
		ln=$($IPTn -nL PREROUTING --line-numbers|grep $MUD)
		n=${ln%% *}
		[ "$n" -gt "0" ]&& $IPTn -D PREROUTING $n
	done
}

case "$1" in

INIT)
#Create upnp nat chain
$IPTn -N $MUD
#Flush upnp chain - in case it already exist
$IPTn -F $MUD
#Add incoming WAN upnp divert
DIVERT
#Create filter upnp chain
$IPT -N $MUD
#Flush upnp filter chain
$IPT -F $MUD
# Check for default DROP or ACCEPT (DMZ)
ln=$($IPT -nL FORWARD --line-numbers|grep all|grep -v state)
n=${ln%% *}
#Add upnp divert
if [ "$n" -gt "0" ];then #insert so goes above DROP/DMZ
	$IPT -I FORWARD $n  -i $WIF -o ! $WIF -j $MUD
else
	$IPT -A FORWARD -i $WIF -o ! $WIF -j $MUD
fi
#Add ip-dn and up scripts to change the primary iptables divert
cat >>$DNS<<EOF1
#!/bin/sh
$LGR "ip-down $ME remove divert"
$FP DN \$1 \$4 &
EOF1

cat >>$UPS<<EOF2
#!/bin/sh
$LGR "ip-up $ME update divert"
$FP UP \$1 \$4 &
EOF2
chmod +x $DNS $UPS
# add default block on low number ports by upnp
if [ ! -r "$CNF" ] || ! grep deny $CNF; then
    echo "deny 0-1024 0.0.0.0/0 0-1024" >>$CNF
fi
$LGR $MUD initialized
;;

UP)
DIVERT
LG "divert replaced"
;;

DN)
UNDIVERT
LG "divert removed"
;;

EXIT)
#Flush upnp nat chain
$IPTn -F $MUD
#remove primary divert
UNDIVERT
#Delete upnp chain
$IPTn -X $MUD
#Flush upnp filter chain
$IPT -F $MUD
#Remove upnp divert
$IPT -D FORWARD -i $WIF -o ! $WIF -j $MUD
#clean up any extra entries
while $IPT -nL FORWARD|grep $MUD
do
	ln=$($IPT -nL FORWARD --line-numbers|grep $MUD)
	n=${ln%% *}
	[ "$n" -gt "0" ]&& $IPT -D FORWARD $n
done
#Delete filter upnp chain
$IPT -X $MUD

#Remove lines from ip-up & down scripts
FOO='/var/mytmp'
cp $DNS $FOO
cat $FOO|grep -v $ME>$DNS
cp $UPS $FOO
cat $FOO|grep -v $ME>$UPS
rm $FOO

LG $MUD uninstalled
;;

*) EX "usage: INIT/EXIT/UP/DN WIF WIP"
;;
esac
