//-------------------------------------------------------------------------------------
// Copyright 2005, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
// 
// 1. NSP PR Framework. 
// 
// THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
// "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS 
// OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY 
// PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR 
// DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, 
// COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. 
// See The GNU General Public License for more details.
//
// These changes are covered under version 2 of the GNU General Public License, 
// dated June 1991.
//-------------------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_prext.h>

static void init(struct ipt_entry_match *m, unsigned int *nfcache) 
{
}

static void help(void) 
{
	printf(
"TI Custom PR Extension match v%s\n"
"[!] --prext value		Match with numerical value\n"
, IPTABLES_VERSION
);
}

static struct option opts[] = {
	{ "prext", 1, 0, '1' },
	{ 0 }
};

static void
parse_prext(const unsigned char *s, struct ipt_prext_info *dinfo)
{
	unsigned int prext;
       
	if (string_to_number(s, 0, 255, &prext) == -1)
		exit_error(PARAMETER_PROBLEM,
			   "Invalid prext `%s'\n", s);

    if (prext > 255)
		exit_error(PARAMETER_PROBLEM,
			   "prext `%d` out of range\n", prext);

   	dinfo->prext = (u_int8_t )prext;
   	return;
}

static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      unsigned int *nfcache,
      struct ipt_entry_match **match)
{
	struct ipt_prext_info *dinfo
		= (struct ipt_prext_info *)(*match)->data;

	switch (c) {
	case '1':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
			           "PREXT match: Only use --prext ONCE!");
		parse_prext(argv[optind-1], dinfo);
		*flags = 1;
		break;

	default:
		return 0;
	}

	return 1;
}

static void
final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
		           "PREXT match: Parameter --prext is required");
}

static void
print_prext(u_int8_t prext, int numeric)
{
 	printf("0x%02x ", prext);
}

/* Prints out the matchinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_match *match,
      int numeric)
{
	const struct ipt_prext_info *dinfo =
		(const struct ipt_prext_info *)match->data;
	printf("PREXT match ");
	print_prext(dinfo->prext, numeric);
}

/* Saves the union ipt_matchinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
{
	const struct ipt_prext_info *dinfo =
		(const struct ipt_prext_info *)match->data;

	printf("--prext ");
	print_prext(dinfo->prext, 1);
}

static
struct iptables_match prext
= { NULL,
    "prext",
    IPTABLES_VERSION,
    IPT_ALIGN(sizeof(struct ipt_prext_info)),
    IPT_ALIGN(sizeof(struct ipt_prext_info)),
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    &save,
    opts
};

void _init(void)
{
	register_match(&prext);
}

