/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_sys_wakeup_ctrl.c
    \brief  PAL wakeup control code file
	

    \author     Ajay Singh
    \version    0.1
 */
#include <pal.h>

/***********************************************************************
 *    Wakeup Control Module for TNETV1050 Communication Processor
 ***********************************************************************/

#define AVALANCHE_WAKEUP_POLARITY_BIT   16

void PAL_sysWakeupCtrl(PAL_SYS_WAKEUP_INTERRUPT_T wakeup_int,
                         PAL_SYS_WAKEUP_CTRL_T      wakeup_ctrl,
                         PAL_SYS_WAKEUP_POLARITY_T  wakeup_polarity)
{
    volatile unsigned int *wakeup_status_reg = (unsigned int*) AVALANCHE_WAKEUP_CTRL_WKCR;

    /* enable/disable */
    if (wakeup_ctrl == WAKEUP_ENABLED)
	{
        /* enable wakeup */
        *wakeup_status_reg |= wakeup_int;
	}
    else
	{
        /* disable wakeup */
        *wakeup_status_reg &= (~wakeup_int);
	}

    /* set polarity */
    if (wakeup_polarity == WAKEUP_ACTIVE_LOW)
	{
        *wakeup_status_reg |=  (wakeup_int << AVALANCHE_WAKEUP_POLARITY_BIT);
	}
    else
	{
        *wakeup_status_reg &= ~(wakeup_int << AVALANCHE_WAKEUP_POLARITY_BIT);
	}
}

