/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/


/*******************************************************************************
 * FILE PURPOSE:    SSP Module Sequence Map
 *******************************************************************************
 * FILE NAME:       ssp_seqmap.c
 *
 * DESCRIPTION:     
 * This file contains the sequence map for the SSP. This map is put in a 
 * separate file to enable easy customization  of the map. The user can change 
 * the map to suit his specific requirements. By default a generic map catering 
 * to the common minimum will be provided. 
 *
 * REVISION HISTORY:
 * 14 Mar 03 - Creation - PSP TII  
 *
 ******************************************************************************/

#include "haltypes.h"

/*
 * Define the masks for sequencer
 * ram registers.
 */
#define SSP_HAL_CLK_HIGH                                 (1 << 0)
#define SSP_HAL_CLK_LOW                                  (0 << 0)
#define SSP_HAL_DATA_HIGH                                (1 << 1)
#define SSP_HAL_DATA_LOW                                 (0 << 1)
#define SSP_HAL_CS_HIGH                                  (1 << 2)
#define SSP_HAL_CS_LOW                                   (0 << 2)
#define SSP_HAL_INPUT_MODE                               (0 << 3)                        
#define SSP_HAL_OUTPUT_MODE                              (1 << 3)                        
#define SSP_HAL_DATA_REG                                 (1 << 4)
#define SSP_HAL_ADDR_REG                                 (0 << 4)
#define SSP_HAL_OPCODE_DIRECT                            ((0x0) << 5)
#define SSP_HAL_OPCODE_TOGGLE                            ((0x1) << 5)
#define SSP_HAL_OPCODE_SHIFT                             ((0x2) << 5)
#define SSP_HAL_OPCODE_BRANCH0                           ((0x4) << 5)
#define SSP_HAL_OPCODE_BRANCH1                           ((0x5) << 5)
#define SSP_HAL_OPCODE_BRANCH                            ((0x6) << 5)
#define SSP_HAL_OPCODE_STOP                              ((0x7) << 5)
#define SSP_HAL_NOT_APPLICABLE                           0

#define SSP_HAL_BRANCH_ADDR_BIT_POS                      8
#define SSP_HAL_CNT_VALUE_BIT_POS                        8

#define SSP_HAL_BRANCH(addr)             ((addr)<<SSP_HAL_BRANCH_ADDR_BIT_POS)

#define SSP_HAL_COUNT(cycles)            ((cycles)<<SSP_HAL_CNT_VALUE_BIT_POS)


UINT32 ssp_halp_spi_write_seq_map[] = 
{
    (SSP_HAL_COUNT(15)      |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_ADDR_REG
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_NOT_APPLICABLE
    ), 

    (SSP_HAL_NOT_APPLICABLE |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW
    ),
    0,0,0,0,0,0,0
};

UINT32 ssp_halp_spi_read_seq_map[] = 
{
    (SSP_HAL_COUNT(15)      |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_ADDR_REG
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_NOT_APPLICABLE
    ),  
    
    (SSP_HAL_NOT_APPLICABLE |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW
    ),
    0,0,0,0,0,0,0
};

UINT32 ssp_halp_spi_stop_seq_map[] = 
{
    (SSP_HAL_NOT_APPLICABLE |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_HIGH
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW
    ),
    0,0,0,0,0,0
};

UINT32 ssp_halp_i2c_seq_map[] ={
    
    /* 0x00 i2c bus idle */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_CLK_HIGH    
    ), 
    
    /* 0x01 i2c stop */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW     
    ),
    
    /* 0x02 */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_HIGH    
    ),
    
    /* 0x03 normal stop  */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_CLK_HIGH      
    ),  

    /* read current byte */

    /* 0x04 i2c start */
    ( (0x01 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_HIGH      
    ),  
    
    /* 0x05 device address shift */
    ( (0x0f <<8)            |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_ADDR_REG 
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_NOT_APPLICABLE
    ),  
    
     /* 0x06 wait for ack */
    ( (0x10 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_LOW       
    ),
    
    /* 0x07 branch to stop on   no ack */
    ( (0x01 <<8)            |SSP_HAL_OPCODE_BRANCH1      |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH      
    ),  
    
     /* 0x08 pause */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH      
    ),
    
    /* 0x09 read in data byte */
    ( (0x0f <<8)            |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_DATA_REG 
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_LOW       
    ),  
    
    /* 0x0A give ack */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_CLK_LOW       
    ),  
    
     /* 0x0B I2C pause */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW       
    ),   

    /* i2c write byte */
    
    /* 0x0C i2c start */
    ( (0x01 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_HIGH      
    ),  
    
    /* 0x0D write device address byte */
    ( (0x0f <<8)            |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_ADDR_REG 
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_LOW       
    ),  
    
    /* 0x0E wait for ack */
    ( (0x10 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_LOW       
    ),  
    
    /* 0x0F branch on no ack */
    ( (0x01 <<8)            |SSP_HAL_OPCODE_BRANCH1      |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH      
    ),  
    
    /* 0x10 I2C pause */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH      
    ),   
        
    /* 0x11 write byte */
    ( (0x0f <<8)            |SSP_HAL_OPCODE_SHIFT        |SSP_HAL_ADDR_REG   
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_NOT_APPLICABLE 
    ), 
    
    /* 0x12 wait for ack */
    ( (0x10 <<8)            |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_LOW        
    ), 
    
    /* 0x13 branch on no ack */
    ( (0x01 <<8)            |SSP_HAL_OPCODE_BRANCH1      |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH       
    ), 
    
    /* 0x14 I2C pause */
    ( (0x00 <<8)            |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_INPUT_MODE          |SSP_HAL_CS_LOW 
                            |SSP_HAL_NOT_APPLICABLE      |SSP_HAL_CLK_HIGH       
    ), 
    
    /* 0x15 i2c BR to stop */
    ( (0x02 <<8)            |SSP_HAL_OPCODE_BRANCH       |SSP_HAL_NOT_APPLICABLE     
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_CS_LOW 
                            |SSP_HAL_DATA_LOW            |SSP_HAL_CLK_LOW       
    )  
};

/* LCD control */
UINT32 ssp_halp_lcd_up_seq_map[] = 
{
    (SSP_HAL_COUNT(9)       |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_LOW            |SSP_HAL_NOT_APPLICABLE
    ),  
    
    (SSP_HAL_COUNT(0)       |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_NOT_APPLICABLE
    ),  
    
    (SSP_HAL_NOT_APPLICABLE |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_NOT_APPLICABLE
    ),
    0,0,0,0,0,0,0
};

UINT32 ssp_halp_lcd_down_seq_map[] = 
{
    (SSP_HAL_COUNT(49)      |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_LOW            |SSP_HAL_NOT_APPLICABLE
    ),  
    
    (SSP_HAL_COUNT(0)       |SSP_HAL_OPCODE_DIRECT       |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_NOT_APPLICABLE
    ),  
    
    (SSP_HAL_NOT_APPLICABLE |SSP_HAL_OPCODE_STOP         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_OUTPUT_MODE         |SSP_HAL_NOT_APPLICABLE
                            |SSP_HAL_DATA_HIGH           |SSP_HAL_NOT_APPLICABLE
    ),
    0,0,0,0,0,0,0
};
