/*
 * $Id: avalanche-flash.c,v 1.1.1.1.6.1 2003/09/03 18:13:29 jaa Exp $
 *
 * Normal mappings of chips in physical memory
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#include <linux/mtd/partitions.h>
#include <asm/avalanche/generic/pal.h>
#include <asm/avalanche/generic/nsp_single_image.h>

#define WINDOW_ADDR CONFIG_MTD_AVALANCHE_START
#define WINDOW_SIZE CONFIG_MTD_AVALANCHE_LEN
#define BUSWIDTH CONFIG_MTD_AVALANCHE_BUSWIDTH

#define AVALANCHE_FLASH2_START           0x1d000000


#if defined(CONFIG_MIPS_TNETV1050SDB) || defined(CONFIG_MIPS_TNETV1060SDB_WR)
#define NUM_OF_MTD_DEVICES  2
#else
#define NUM_OF_MTD_DEVICES  1
#endif

#if defined CONFIG_MIPS_TNETV1050VTAM
#define NUM_OF_MTD_DEVICES 1
#endif 

/* avalanche__partition_info gives details on the logical partitions that splits
 * the flash device into. If the size if zero we use up to the end of
 * the device. */
#define MAX_NUM_PARTITIONS 10

#include <asm/mips-boards/prom.h>

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
static devfs_handle_t devfs_dir_handle;		/* Base dir */
static devfs_handle_t devfs_dir1_handle;	/* Primary dir */
static devfs_handle_t devfs_dir2_handle;	/* Secondary dir */
static void avalanche_mtd_notify_add(struct mtd_info* mtd);
static void avalanche_mtd_notify_remove(struct mtd_info* mtd);

static struct mtd_notifier notifier = {
	add:	avalanche_mtd_notify_add,
	remove:	avalanche_mtd_notify_remove,
};
#endif


struct avalanche_chip_info
{
    char *name;
    int addr;
    int size;
    int buswidth;
};

struct avalanche_mtd
{
    struct avalanche_chip_info *avalanche_chip_info;
    struct mtd_partition avalanche_partition_info[MAX_NUM_PARTITIONS];
    struct mtd_info *avalanche_mtd_info; 
    struct map_info avalanche_map;
    int num_of_partitions;
};

extern char *prom_getenv(char *name);
static int create_mtd_partitions(void);
static void __exit avalanche_mtd_cleanup(void);

struct avalanche_chip_info avalanche_chip_info[NUM_OF_MTD_DEVICES] = {   
	{ "Physically mapped flash:0",WINDOW_ADDR, WINDOW_SIZE, BUSWIDTH },
#if defined(CONFIG_MIPS_TNETV1050SDB) || defined(CONFIG_MIPS_TNETV1060SDB_WR)
	{ "Physically mapped flash:1",AVALANCHE_FLASH2_START, WINDOW_SIZE, BUSWIDTH },
#endif
};                                                                  

struct avalanche_mtd avalanche_mtd[NUM_OF_MTD_DEVICES];
int avalanche_mtd_ready=0;

__u8 avalanche_read8(struct map_info *map, unsigned long ofs)
{
	return __raw_readb(map->map_priv_1 + ofs);
}

__u16 avalanche_read16(struct map_info *map, unsigned long ofs)
{
	return __raw_readw(map->map_priv_1 + ofs);
}

__u32 avalanche_read32(struct map_info *map, unsigned long ofs)
{
	return __raw_readl(map->map_priv_1 + ofs);
}

void avalanche_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	memcpy_fromio(to, map->map_priv_1 + from, len);
}

void avalanche_write8(struct map_info *map, __u8 d, unsigned long adr)
{
	__raw_writeb(d, map->map_priv_1 + adr);
	mb();
}

void avalanche_write16(struct map_info *map, __u16 d, unsigned long adr)
{
	__raw_writew(d, map->map_priv_1 + adr);
	mb();
}

void avalanche_write32(struct map_info *map, __u32 d, unsigned long adr)
{
	__raw_writel(d, map->map_priv_1 + adr);
	mb();
}

void avalanche_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
	memcpy_toio(map->map_priv_1 + to, from, len);
}


int avalanche_flash_init(struct avalanche_mtd *dev)
{
	struct avalanche_chip_info *chip_info = dev->avalanche_chip_info;
	struct map_info * avalanche_map = &dev->avalanche_map;
	struct mtd_info * avalanche_mtd_info;
        
        avalanche_map->name = chip_info->name;
        avalanche_map->size = chip_info->size;
        avalanche_map->buswidth = chip_info->buswidth;
        avalanche_map->read8 = avalanche_read8;
        avalanche_map->read16 = avalanche_read16;
        avalanche_map->read32 = avalanche_read32;
        avalanche_map->copy_from = avalanche_copy_from;
        avalanche_map->write8 = avalanche_write8;
        avalanche_map->write16 = avalanche_write16;
        avalanche_map->write32 = avalanche_write32;
        avalanche_map->copy_to = avalanche_copy_to;

       	printk(KERN_NOTICE "avalanche flash device: 0x%lx at 0x%lx.\n",
				(unsigned long) chip_info->size,
				(unsigned long)chip_info->addr);
	avalanche_map->map_priv_1 = (unsigned long)ioremap_nocache(chip_info->addr, chip_info->size);

	if (!avalanche_map->map_priv_1) {
		printk("Failed to ioremap\n");
		return -EIO;
	}

	avalanche_mtd_info = do_map_probe("cfi_probe", avalanche_map);
	if (!avalanche_mtd_info)
	{
		printk("registering mtd failed\n");
		return -ENXIO;
	}

	dev->avalanche_mtd_info = avalanche_mtd_info;	
	avalanche_mtd_info->module = THIS_MODULE;
	

#if 0
	if (!(num_of_partitions = create_mtd_partitions(dev)));
		add_mtd_device(avalanche_mtd_info);
	else		
		add_mtd_partitions(avalanche_mtd_info, dev->avalanche_partition_info, num_of_partitions);

#endif	
	return 0;	
}

#ifdef CONFIG_DEVFS_FS
/* This function hooks up to the mtd notify callbacks. When we add a
   mtd partion, this function looks at the name, and creates a symlink from
   the /dev/ti/ directory to the proper /dev/mtdblock/ entry. This way, all
   accesses through the system can be done without worrying about the mtd
   number. We can't do this before running add_partition, as we don't know
   the index number yet. */
static void avalanche_mtd_notify_add(struct mtd_info* mtd)
{
	devfs_handle_t dir_handle;	/* Primary dir */
	char link_name[50];
	char *link;
	char *link_source;
	int isroot=0;

	if(!mtd)		return;
	if(!mtd->name)		return;
	if(mtd->index>20)	return;

	/* We cheat on the link name. If the name is in the top
	   dir, we point to link_name[3]. This way, we don't waste space */
	sprintf(link_name, "../../mtdblock/%u", mtd->index);
	link=link_name+3;
	link_source=NULL;

	/* Walk through the set names, and if found, set to the expected values */
	dir_handle=devfs_dir_handle;
	if(!strcmp("RESERVED_PRIMARY_ROOT_FS", mtd->name)) {
		link=link_name;		/* Use full path */
		dir_handle=devfs_dir1_handle;
		link_source="root"; isroot=1;
	} else if(!strcmp("RESERVED_PRIMARY_KERNEL", mtd->name)) {
		link=link_name;		/* Use full path */
		dir_handle=devfs_dir1_handle;
		link_source="kernel";
	} else if(!strcmp("RESERVED_PRIMARY_IMAGE", mtd->name)) {
		link_source="active_image";
	} else if(!strcmp("RESERVED_SECONDARY_IMAGE", mtd->name)) {
		link_source="inactive_image";
	} else if(!strcmp("RESERVED_PRIMARY_XML_CONFIG", mtd->name)) {
		link_source="active_config";
	} else if(!strcmp("RESERVED_SECONDARY_XML_CONFIG", mtd->name)) {
		link_source="inactive_config";
	} else if(!strcmp("RESERVED_BOOTLOADER", mtd->name)) {
		link_source="bootloader";
	}

	/* Otherwise, just add the partition with the given name in the
	   primary partition*/
	if(!link_source){
		link=link_name;		/* Use full path */
		dir_handle=devfs_dir1_handle;
		link_source=mtd->name;
	}

	printk("Avalanche:      linking partition %s: %s -> %s\n",
				mtd->name, link_source, link_name);

	/* Now, just do the link */
	devfs_mk_symlink(dir_handle, link_source, DEVFS_FL_DEFAULT,
				link, NULL, NULL);

	/* HACK to get the root filesystem to be really linked here */
	if(isroot){
		/* Do nothing yet. TODO: Lets find a way */
	}
}

static void avalanche_mtd_notify_remove(struct mtd_info* mtd)
{
	if (!mtd)
		return;
}
#endif


int __init avalanche_mtd_init(void)
{      
	int i;
	
	for(i=0;i <NUM_OF_MTD_DEVICES; i++) {
		int ret;

		avalanche_mtd[i].avalanche_chip_info = &avalanche_chip_info[i];
		ret = avalanche_flash_init(avalanche_mtd + i);	
		
		if(ret !=0)
			printk("failed initializing flash dev %d\n",i);
	}

	/* Create the TI directory for the real partition names */
	devfs_dir_handle = devfs_mk_dir(NULL, "ti_partitions", NULL);
	devfs_dir1_handle = devfs_mk_dir(devfs_dir_handle, "active", NULL);
	devfs_dir2_handle = devfs_mk_dir(devfs_dir_handle, "inactive", NULL);
	register_mtd_user(&notifier);

	create_mtd_partitions(); 
	/* Avalanche flash is initialized */
	avalanche_mtd_ready=1;

	return 0;
}


static char *strdup(char *str)
{
	int n = strlen(str)+1;
	char *s = kmalloc(n, GFP_KERNEL);
	if (!s) return NULL;
	return strcpy(s, str);
}

static int avalanche_check_mtd(struct avalanche_mtd *dev, unsigned int flash_base, unsigned int flash_end)
{
	int window_start = dev->avalanche_chip_info->addr;
        int window_size = dev->avalanche_chip_info->size;

	flash_base = (unsigned int)virt_to_bus((char *)flash_base);	
	flash_end = (unsigned int)virt_to_bus((char *)flash_end);

#if 0	
	printk("flash base = %x, flash_end = %x, window_start = %x, window_end = %x\n",flash_base,flash_end,window_start,window_size);
#endif

	if( (flash_base >= window_start && flash_base <= (window_start + window_size) )  &&
		 (flash_end >= window_start && flash_end <= (window_start + window_size)) )
        		return 1;

	return 0;

}

static void avalanche_add_partition(char * env_name, char * flash_base, char * flash_end)
{
	
	int i = 0;
	int found =0;
	int mtd_partition;
	int offset;
	int size;
		
	struct avalanche_mtd *dev =NULL;	
        struct mtd_partition *avalanche_partition_info;
        struct avalanche_chip_info *avalanche_chip_info;
 	struct mtd_info *avalanche_mtd_info;

	for(i = 0; i< NUM_OF_MTD_DEVICES; i++)
	{
		dev = &avalanche_mtd[i];
		if( !avalanche_check_mtd(dev,(unsigned int)flash_base, (unsigned int)flash_end))
		{
			continue;
		}
		found = 1;
		break;
	}
                        
 	if(!found)
		return;

       	avalanche_partition_info = dev->avalanche_partition_info;
        avalanche_chip_info = dev->avalanche_chip_info;
	avalanche_mtd_info = dev->avalanche_mtd_info;

	offset = virt_to_bus(flash_base) - avalanche_chip_info->addr;
	size = flash_end - flash_base;
	
	printk("Avalanche: Adding a %s image (0x%x), with size (0x%x).\n",env_name, offset, size);

	/* Setup the partition info. We duplicate the env_name for the partiton name */
	mtd_partition = dev->num_of_partitions;
	avalanche_partition_info[mtd_partition].name = strdup(env_name);
	avalanche_partition_info[mtd_partition].offset = offset;
	avalanche_partition_info[mtd_partition].size = size;
	avalanche_partition_info[mtd_partition].mask_flags = 0;
	add_mtd_partitions(avalanche_mtd_info, &avalanche_partition_info[mtd_partition], 1);
	dev->num_of_partitions++;
}

/* This takes an env name, and parses its contents into the addresses contained inside.
   It only supports 2 addresses.
   TODO: support a single address
 */
static int	avalanche_parse_env_address(char *env_name, unsigned char **flash_base,
				unsigned char **flash_end)
{
	char	image_name[30];
	char	*env_ptr;
	char	*base_ptr;
	char	*end_ptr;

	/* Get the image variable */
	env_ptr = prom_getenv(env_name);
	if(!env_ptr){
		printk("Avalanche: invalid env name, %s.\n", env_name);
		return -1;	/* Error, no image variable */
	}
	strncpy(image_name, env_ptr, 30);
	image_name[29]=0;

	/* Extract the start and stop addresses of the partition */
	base_ptr = strtok(image_name, ",");
	end_ptr = strtok(NULL, ",");
	if ((base_ptr == NULL) || (end_ptr == NULL)) {	
		printk("Avalanche: Couldn't tokenize %s start,end.\n", image_name);
		return -1;
	}

	*flash_base = (unsigned char*) simple_strtol(base_ptr, NULL, 0);
	*flash_end = (unsigned char*) simple_strtol(end_ptr, NULL, 0);
	if((!*flash_base) || (!*flash_end)) {
		printk("Avalanche: Unable to convert :%s: :%s: into start,end values.\n",
				env_name, image_name);
		return -1;
	}

	return 0;
}

static int	avalanche_get_single_image(char *bootcfg_name, unsigned char **flash_base,
				unsigned char **flash_end)
{
	char	*env_ptr;
	char	*base_ptr;
	char	*end_ptr;
	char	image_name[30];

	if(!bootcfg_name || !flash_base || !flash_end)
		return -1;

	env_ptr = prom_getenv(bootcfg_name);
	if(!env_ptr){
		printk("Avalanche: %s variable not found.\n", bootcfg_name);
		return -1;	/* Error, no bootcfg variable */
	}

	/* Save off the image name */
	strncpy(image_name, env_ptr, 30);
	image_name[29]=0;

	end_ptr=strtok(image_name, "\"");
	base_ptr=strtok(NULL, "\"");		/* Loose the last " */
	if(!end_ptr || !base_ptr){
		printk("Avalanche: invalid bootcfg format, %s.\n", image_name);
		return -1;	/* Error, invalid bootcfg variable */
	}

	/* Now, parse the addresses */
	return avalanche_parse_env_address(base_ptr, flash_base, flash_end);
}

/* Ok, we have a single image on flash. Lets parse the header, and setup the partitions.
   After that, then lets look for the other cool things to add to the system */
//static int	avalanche_single_image_parse_partitions(unsigned char *flash_base, unsigned char *flash_end)
static int avalanche_do_single_image(void)
{
	struct nsp_img_hdr		*hdr;
	struct nsp_img_hdr_section_info	*sect_info;
	struct nsp_img_hdr_sections	*section;
	unsigned char			*flash_base, *flash_end;
	unsigned char			*start, *end;
	char				*name;
	char				*config;
	char				*config_2;
	int	i;
	int	total_sects=0;

	/* Get the bootcfg env variable first */
	if(avalanche_get_single_image("BOOTCFG", &flash_base, &flash_end)) {
		/* Error, fallback */
		return -1;
	}

	/* Get access to the header, and do some validation checks */
	hdr=(struct nsp_img_hdr*)flash_base;
	if(hdr->head.magic != NSP_IMG_MAGIC_NUMBER)
		return -1;	/* Not a single image */

	/* Get access to the other pieces */
	sect_info=(struct nsp_img_hdr_section_info*)(flash_base + hdr->head.sect_info_offset);
	section=(struct nsp_img_hdr_sections*)(flash_base + sect_info->sections_offset);

	/* Look for the root fs, and add it first. This way we KNOW where the rootfs is */
	for(i=0; i< sect_info->num_sects && i<MAX_NUM_PARTITIONS; i++, section++){
		/* Add only the root partition */
		if(section->type != NSP_IMG_SECTION_TYPE_FILESYSTEM_ROOT){
			continue;
		}
		start=flash_base + section->offset;
		end=start + section->total_size;
		avalanche_add_partition(section->name, start, end);
		total_sects++;
	}

	section=(struct nsp_img_hdr_sections*)(flash_base + sect_info->sections_offset);
	/* Ok, lets add the partitions. Yes, this double adds the root fs */
	for(i=0; i< sect_info->num_sects && i<MAX_NUM_PARTITIONS; i++, section++){

		/* we hardcode the kernel and root fs. This keeps things simple */
		name=section->name;
		if(section->type == NSP_IMG_SECTION_TYPE_FILESYSTEM_ROOT)
			name = "RESERVED_PRIMARY_ROOT_FS";
		else if(section->type == NSP_IMG_SECTION_TYPE_KERNEL)
			name = "RESERVED_PRIMARY_KERNEL";

		start=flash_base + section->offset;
		end=start + section->total_size;
		avalanche_add_partition(name, start, end);
		total_sects++;
	}

	/* Next, lets add the single image */
	avalanche_add_partition("RESERVED_PRIMARY_IMAGE", flash_base, flash_end);
	total_sects++;

	/* Ok. Check now if we are dual image. If yes, lets mount the
	   secondary image */
	name=NULL; config=NULL; config_2=NULL;
	if(!avalanche_get_single_image("BOOTCFG_A", &start, &end)){
		/* We have an "A" image. Check if it matches */
		if(flash_base==start && flash_end==end){
			name="BOOTCFG_B";	/* Secondary is B */
			config="CONFIG_A";	/* Config is A */
			config_2="CONFIG_B";	/* 2nd Config is B */
		}
	}
	if(!avalanche_get_single_image("BOOTCFG_B", &start, &end)){
		/* We have an "B" image. Check if it matches */
		if(flash_base==start && flash_end==end){
			name="BOOTCFG_A";	/* Secondary is A */
			config="CONFIG_B";	/* Config is B */
			config_2="CONFIG_A";	/* 2nd Config is A */
		}
	}

	/* If name was assigned above, then get the address, and add the
	   partition */
	if(name && !avalanche_get_single_image(name, &start, &end)) {
		avalanche_add_partition("RESERVED_SECONDARY_IMAGE", start, end);
		total_sects++;
	}

	/* Lets add the xml config partition. First check if we have a specific
	   config for this image, if yes, use it, if no, or invalid, fallback to
	   CONFIG */
	if(config && !avalanche_parse_env_address(config, &start, &end)){
		avalanche_add_partition("RESERVED_PRIMARY_XML_CONFIG", start, end);
		total_sects++;
	} else if (!avalanche_parse_env_address("CONFIG", &start, &end)){
		avalanche_add_partition("RESERVED_PRIMARY_XML_CONFIG", start, end);
		total_sects++;
	}

	/* Lets add the secondary partition */
	if(config_2 && !avalanche_parse_env_address(config_2, &start, &end)){
		avalanche_add_partition("RESERVED_SECONDARY_XML_CONFIG", start, end);
		total_sects++;
	}

	if (!avalanche_parse_env_address("BOOTLOADER", &start, &end)){
		avalanche_add_partition("RESERVED_BOOTLOADER", start, end);
		total_sects++;
	}

	return total_sects;
}

static int create_mtd_partitions(void)
{
	unsigned char *flash_base;
	unsigned char *flash_end;
	char *env_ptr;
	char *base_ptr;
	char *end_ptr;
        int num_of_partitions = 0;
	int ret;

	/* Try to use the single image header */
	ret=avalanche_do_single_image();
	if(ret>0)
		return ret;

	printk("Avalanche: No single image format found. Using mtd env variables\n");
	do {
		char	env_name[20];
		char	image_name[30];


		/* get base and end addresses of flash file system from environment */
		sprintf(env_name, "mtd%1u", num_of_partitions);
		printk("Looking for mtd device :%s:\n", env_name);
		env_ptr = prom_getenv(env_name);
		if(env_ptr == NULL) {
			/* No more partitions to find */
			break;
		}

		/* Save off the value into writable storage */
		strncpy(image_name, env_ptr, 30);
		image_name[29]=0;

		/* Extract the start and stop addresses of the partition */
		base_ptr = strtok(image_name, ",");
		end_ptr = strtok(NULL, ",");
		if ((base_ptr == NULL) || (end_ptr == NULL)) {	
			printk("ERROR: Invalid %s start,end.\n", env_name);
			break;
		}

		flash_base = (unsigned char*) simple_strtol(base_ptr, NULL, 0);
		flash_end = (unsigned char*) simple_strtol(end_ptr, NULL, 0);
		if((!flash_base) || (!flash_end)) {
			printk("ERROR: Invalid %s start,end.\n", env_name);
			break;
		}
                
		avalanche_add_partition(env_name,flash_base,flash_end);
		num_of_partitions++;

	} while (num_of_partitions < MAX_NUM_PARTITIONS);

	return num_of_partitions;
}

static void __exit avalanche_mtd_cleanup(void)
{
	int i;

	avalanche_mtd_ready=0;
	for(i=0;i <NUM_OF_MTD_DEVICES; i++)
	{
		struct mtd_info *avalanche_mtd_info = avalanche_mtd[i].avalanche_mtd_info;		
		struct map_info *avalanche_map = &avalanche_mtd[i].avalanche_map;		

		if (avalanche_mtd_info) {
			del_mtd_partitions(avalanche_mtd_info);
			del_mtd_device(avalanche_mtd_info);
			map_destroy(avalanche_mtd_info);
		}

		if (avalanche_map->map_priv_1) {
			iounmap((void *)avalanche_map->map_priv_1);
			avalanche_map->map_priv_1 = 0;
		}
	}
	unregister_mtd_user(&notifier);
}

module_init(avalanche_mtd_init);
module_exit(avalanche_mtd_cleanup);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Avalanche CFI map driver");
