/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/


#include "_tistdtypes.h"
#include "pal_defs.h"
#include "ddc_cpmacDrv.h"

#include "cpswhalcommon_stddef.h"
#include "cpswhalcommon_miimdio.h"

#include "ddc_switch.h"
#include "csl_switchPrvt.h"
#ifdef VX_COMPILATION
#include "vxWorkspal.h"
#endif
#include "ddc_cpmacDrv.h"


#define ICPLUS_175C_PHYRESET_SHIFT_MASK    15
#define ICPLUS_175C_PHYRESET_MASK          0x1

#define ICPLUS_175C_STATUS_SHIFT_MASK   0
#define ICPLUS_175C_STATUS_MASK         0x3

#define ICPLUS_175C_LINK_SHIFT_MASK     2
#define ICPLUS_175C_LINK_MASK           0x1

#define ICPLUS_175C_DUPLEX_SHIFT_MASK   8
#define ICPLUS_175C_DUPLEX_MASK         0x1

#define ICPLUS_175C_DUPLEX_SET_SHIFT_MASK   8

#define ICPLUS_175C_SPEED_SHIFT_MASK    13
#define ICPLUS_175C_SPEED_MASK          0x1

#define ICPLUS_175C_SPEED_SET_SHIFT_MASK   13

#define ICPLUS_175C_FC_SHIFT_MASK       15
#define ICPLUS_175C_FC_MASK             0x1

#define ICPLUS_175C_AUTO_ENABLE_SHIFT_MASK      12
#define ICPLUS_175C_AUTO_ENABLE_MASK            0x1

#define ICPLUS_175C_AUTO_RESTART_SHIFT_MASK     9
#define ICPLUS_175C_AUTO_RESTART_MASK           0x1



/* general commands*/
#define PORT_PHY_ADDR(portNum)      (portNum -1)
#define PORT_REG_ADDR(portNum,basePortNum) (portNum + basePortNum)
#define VLAN_PHY_ADDR(portNum)    0x1E

PUBLIC PHY_DEVICE * pSwitchInfo;


/*******************************************************************************
*
* cslMarvell6063Start - Does the primary initialization of the switch.
*
* Performs the initialization of the switch and switch specific SW data.
*
* INPUTS:
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslICPlus175CStart (void)
{
    Int32 retVal = -1;
    
    return retVal;
}



/*******************************************************************************
*
* cslICPlus175CRegVal2OperStatus - extract the user operational status out of register value.
*
* This code contains switch specific information. The main purpose is converting switch specific info to 
* the neutral code. As a neutral data types we're using external interfaces.
*
* INPUTS:
*     [In]    OPER_TYPE oper 
*     [In]    Uint16   data
* 
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslICPlus175CRegVal2OperStatus (OPER_TYPE_E oper, Uint16 data)
{
    Int32 retVal = -1;
    
    

    if (oper == OPER_LINK)
    {
        retVal = (Int32)data & (ICPLUS_175C_LINK_MASK << ICPLUS_175C_LINK_SHIFT_MASK);
        if(retVal == 0)
            return PORT_LINK_DOWN;
        return PORT_LINK_UP;
    }

    else if (oper == OPER_DUPLEX)
    {
        retVal = (Int32)data & (ICPLUS_175C_DUPLEX_MASK << ICPLUS_175C_DUPLEX_SHIFT_MASK);
        if(retVal == 0)
            return PORT_DUPLEX_HALF;
        return PORT_DUPLEX_FULL;
    }

    else if (oper == OPER_SPEED)
    {
        retVal = (Int32)data & (ICPLUS_175C_SPEED_MASK << ICPLUS_175C_SPEED_SHIFT_MASK);
        if(retVal == 0)
            return PORT_SPEED_10;
        return PORT_SPEED_100;
    }

    else if (oper == OPER_FC)
    {
        retVal = (Int32)data & (ICPLUS_175C_LINK_MASK << ICPLUS_175C_LINK_SHIFT_MASK);
        if (retVal == 0)
            return PORT_FC_OFF;
        return PORT_FC_ON;
    }
    else if (oper == OPER_AUTO)
    {
        retVal = (Int32)data & (ICPLUS_175C_AUTO_ENABLE_MASK << ICPLUS_175C_AUTO_ENABLE_SHIFT_MASK);
        if (retVal == 0)
            return PORT_AUTO_NO;
        return PORT_AUTO_YES;
    }
    
    return retVal;
}

/*******************************************************************************
*
* cslICPlus175COperStatus2RegVal - constructs the register value to be written out of data and user request
*
* Enable/Disable a selected port
*
* INPUTS:
*     [In]    OPER_TYPE     oper 
*     [In]    Int32        operValue
*     [In]    Uint16       data
* 
*
*
* RETURNS: 
*       The final register value
*
*
*/

Uint16 cslICPlus175COperStatus2RegVal (OPER_TYPE_E oper,Int32 operValue,Uint16 data)
{
    Uint16 retVal = 0xffff;

    
   if (oper == OPER_DUPLEX_SET)
    {
        retVal = data & (~((Uint16)(ICPLUS_175C_DUPLEX_MASK << ICPLUS_175C_DUPLEX_SET_SHIFT_MASK)));
        if(operValue == PORT_DUPLEX_FULL)
            retVal |= (ICPLUS_175C_DUPLEX_MASK << ICPLUS_175C_DUPLEX_SET_SHIFT_MASK);

        /* assert software reset, otherwise the changes will be ignored */
		
        retVal |= (Uint16)(ICPLUS_175C_AUTO_RESTART_MASK << ICPLUS_175C_AUTO_RESTART_SHIFT_MASK);  

    }
    
    else if (oper == OPER_SPEED_SET)
    {
        retVal = data & (~((Uint16)(ICPLUS_175C_SPEED_MASK << ICPLUS_175C_SPEED_SET_SHIFT_MASK)));
        if(operValue == PORT_SPEED_100)
            retVal |= (ICPLUS_175C_SPEED_MASK << ICPLUS_175C_SPEED_SET_SHIFT_MASK);

        /* assert software reset, otherwise the changes will be ignored */
        retVal |= (Uint16)(ICPLUS_175C_AUTO_RESTART_MASK <<ICPLUS_175C_AUTO_RESTART_SHIFT_MASK); 
    }

    else if (oper == OPER_FC)
    {
        retVal = data & (~((Uint16)(ICPLUS_175C_FC_MASK << ICPLUS_175C_FC_SHIFT_MASK)));
        if(operValue == PORT_FC_ON)
            retVal |= (ICPLUS_175C_DUPLEX_MASK << ICPLUS_175C_DUPLEX_SHIFT_MASK);
    }

    else if (oper == OPER_AUTO)
    {
        retVal = data & (~((Uint16)(ICPLUS_175C_AUTO_ENABLE_MASK << ICPLUS_175C_AUTO_ENABLE_SHIFT_MASK)));
        if(operValue != PORT_AUTO_NO)
        {
            retVal |= (Uint16)(ICPLUS_175C_AUTO_ENABLE_MASK << ICPLUS_175C_AUTO_ENABLE_SHIFT_MASK);
            /* need to reset all the settings. if not , we will not able to get the speed and duplex status*/
            retVal |= ((Uint16)(ICPLUS_175C_PHYRESET_MASK << ICPLUS_175C_PHYRESET_SHIFT_MASK));
           
        }
        
        /* assert software reset, otherwise the changes will be ignored */
        retVal |= (Uint16)(ICPLUS_175C_AUTO_RESTART_MASK << ICPLUS_175C_AUTO_RESTART_SHIFT_MASK); 
    }

    else if (oper == OPER_RESET)
    {
        retVal = data | ((Uint16)(ICPLUS_175C_PHYRESET_MASK << ICPLUS_175C_PHYRESET_SHIFT_MASK));
       }

    return retVal;
    
}



/*******************************************************************************
*
* cslICPlus175CSwitchGetVersion - returns the switch specific information
*
* 
*
* INPUTS:
*     [Out]   PSW_VER_INFO pSwitchInfo
* 
*
*
* RETURNS: 
*       Status
*
*
*/

void cslICPlus175CSwitchGetVersion(PSW_VER_INFO pSwitchVerInfo)
{
    Uint32 regaddr,phyaddr,data;

    regaddr = 0x3;

    phyaddr = PORT_PHY_ADDR(0);

    pSwitchVerInfo->is802_1_P_capable      = PAL_True; 
    pSwitchVerInfo->is802_1_Q_capable      = PAL_True;
    pSwitchVerInfo->maxNumberOfPorts       = 6;
    pSwitchVerInfo->maxNumberOfPriorities  = 1;
    pSwitchVerInfo->maxNumberOfVLANs       = 16;

    pSwitchVerInfo->Model = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

}



         
/*******************************************************************************
*
* cslICPlus175CPortNum2RegAddr - returns the register address
*
* Retreive register address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/

Uint32 cslICPlus175CPortNum2RegAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 regAddr = 0;

    switch (type) 
            { 
                case OPER_STATUS: 
                    regAddr = 0x4; 
                    break; 
                case OPER_LINK: 
                    regAddr = 0x1; 
                    break; 
                case OPER_DUPLEX: 
                    regAddr = 0x0; 
                    break; 
                case OPER_SPEED: 
                    regAddr = 0x0; 
                    break; 
                case OPER_AUTO: 
                    regAddr = 0x0; 
                    break; 
                case OPER_FC: 
                    regAddr = 0x4; 
                    break; 
		  case OPER_RESET: 
                    regAddr = 0x0; 
                    break; 
                case OPER_RESET_STAT: 
                    regAddr = 0x1d; 
                    break; 
                default: 
                    break; 
        } 

    return regAddr;
}

/*******************************************************************************
*
* cslICPlus175CPortNum2PHYAddr - returns the PHY address
*
* Retreive PHY address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/


Uint32 cslICPlus175CPortNum2PHYAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 phyAddr = 0;
    
    switch (type) 
        { 
            case OPER_STATUS: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 
            case OPER_LINK: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 
            case OPER_DUPLEX: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 
            case OPER_DUPLEX_SET: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 

            case OPER_SPEED: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 
          
            case OPER_SPEED_SET: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 

	    case OPER_RESET: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 

            case OPER_AUTO: 
                phyAddr = PORT_PHY_ADDR(portNum); 
                break; 
            default: 
                break; 
        }

    return phyAddr;
}

/**************************************************** VLAN submodule **************************************/

#define VLN_DEFAULT_MASK        0xFFFF
#define VLN_DEFAULT_MASK_SHIFT  0

/*******************************************************************************
*
* cslICPlus175CVlanDefaultSet - Defines the default VLAN
*
* Defines the default VLAN. The meaning can change from implementation to implementation.
*
* INPUTS:
*     [In]   Uint32         portNum
*     [In]   PCSL_SWITCH_VLAN_TABLE pVlnEntry
*
*
* RETURNS: 
*       Status
*
*
*/


Int32 cslICPlus175CVlanDefaultSet(Uint32 portNum,PCSL_SWITCH_VLAN_TABLE pVlnEntry)
{
    Uint32 regaddr,phyaddr,data;

    regaddr = PORT_REG_ADDR(portNum, 0x18);
    if(portNum == 5)
    {
		regaddr = 0x1e;
    }
    phyaddr = 0x1d;

     data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

    data = (data & (~(VLN_DEFAULT_MASK << VLN_DEFAULT_MASK_SHIFT))) | (pVlnEntry->vlanId & (VLN_DEFAULT_MASK << VLN_DEFAULT_MASK_SHIFT));

    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

    return PAL_True;
}

/*******************************************************************************
*
* cslICPlus175CVlanUpdate - Updates the VLAN in VLAN table
*
* Writes the VLAN data to the VLAN table.
*
* INPUTS:
*     [In]   PCSL_SWITCH_VLAN_TABLE pVlnEntry
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslICPlus175CVlanUpdate(PCSL_SWITCH_VLAN_TABLE pVlnEntry)
{
	Uint32 regaddr,phyaddr,portMask,portTagMask=0;
	Uint32 data = 0;
	phyaddr = VLAN_PHY_ADDR(0);
	portMask = pVlnEntry->portMask;
	portTagMask = pVlnEntry->portTagMask;

	regaddr = 0x9;
	if(pVlnEntry->valid)
		data |= (1 << 7);
	else
		data &=( ~(1 << 7));
	
	//_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
	
	data = 0;
	
	if((pVlnEntry->vlanId) %2==0)
	{
		regaddr = ((pVlnEntry->vlanId)/2)+1;
		data &= (~(0x3f));
		_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
		if(pVlnEntry->valid)
			data |= portMask;

	}else
	{

		regaddr = ((pVlnEntry->vlanId)+1)/2;
		data &= (~(0x3f00));
		_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
		if(pVlnEntry->valid)
			data |= (portMask << 8);

	}
	if(pVlnEntry->valid)
		_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

	phyaddr = 0x1d;
	regaddr = 0x17;
	data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	data &= (~(0xFFC3));

	_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
	if(pVlnEntry->valid)
	{
		data = 0;
		portTagMask = portTagMask & portMask;
		if(portTagMask & (1 << 5))
		{
			data |= 0x2;
			portTagMask &= (~(1 <<5));
		}


		data |= (portTagMask << 11);
		data |= ((~(portTagMask)) << 6);
		_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

		/*data = 0;

		regaddr = 0x10;

		data &=  (~ (1 << 12 | 1 << 11 | 1 << 10 | 1 << 9 | 1 << 8));
		
		_cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
		*/
	}
    return PAL_True;
}


/*******************************************************************************
*
* cslICPlus175CSpecialCapsSet - sets special capabilities configuration of the switch
*   
*
* INPUTS:
*     [In]   Uint32                 portNum
*     [In]   Uint32                 arg -the value of the specifice capability
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslICPlus175CSpecialCapsSet(Uint32 cmd, Uint32 arg)
{
    Uint32 regaddr,phyaddr,data,port;                    
    INGRESS_VLN_MODE mode = (INGRESS_VLN_MODE)arg;
    switch ((OPER_TYPE_E)cmd)
    {
/* all the ports are ingress checkin/ no ingress checking */

    case OPER_VLAN_TABLE_SET:
        regaddr = 0x13;
        phyaddr = 0x1d;

	data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	data &=  ~(1 << 0 |1 << 2 | 1 << 3 | 1 << 4  | 1 << 9 | 1 << 10 | 1 << 11 | 1 << 12 );

	data |= (1 << 15 | 1 << 8 | 1 << 7 | 1 << 1);

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

	regaddr = 0x14;
	data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	data &=  ~(1 << 0 |1 << 1 | 1 << 2 | 1 << 4  | 1 << 8 | 1 << 9 | 1 << 11 | 1 << 12 );

	data |= (1 << 15 | 1 << 10 | 1 << 7 | 1 << 3 );

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
	
       regaddr = 0x15;

	data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	data &=  ~( 1 << 8 | 1 << 9 | 1 << 10 | 1 << 11 );

	data |= (1 << 15 | 1 << 12);

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

	regaddr = 0x12;
        phyaddr = 0x1e;

	data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	data |= (1 << 7 | 1 << 5 | 1 << 4 | 1 << 3 | 1 << 2 | 1 << 1 | 1 << 0);

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
	 
        break;

    /* nothing to be done*/
    case OPER_INGRESS:  
		regaddr = 0x9;
		phyaddr = 0x1e;
        
           data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
           switch ( mode )
           {
           default:
           case INGRESS_VLN_MODE_8021Q_DISABLE:
              data &= ~(1 << 7);
              break;

           case INGRESS_VLN_MODE_8021Q_CHECK:
              data &= ~(1 << 7);
              break;
           // enable 8021q mode   
           case INGRESS_VLN_MODE_8021Q_SECURE:
              data |= (1 << 7);
              break;
           }

	     _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
          
        break;
	/*nothing to be done*/
    case OPER_INGRESS_OFF:

       break;
	   /* enable special tag feature of the ic plus switch */
    case OPER_PORT_RECOGNITION_ON:   
        phyaddr = 0x1e;
        regaddr = 0x10;

        data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);


	/* disable learning on each port */

	data &= 0xe0ff;
		
	data |= 1 << 7; // enable special tag function

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

	 phyaddr = 0x1d;
        regaddr = 0x17;

	 data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);

	 data  |= (1 << 10 |1 << 9 | 1 << 8 | 1 << 7 | 1 << 6 | 1 << 1 );

	 _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
	
        break;

	/*nothing to be done here*/
    case OPER_TRANSMIT_ON_FIXED_PORT:
    	break;

    /* nothing to be done here*/
    case OPER_VLANTUNNEL:
    default:
            break;           

    }

    return PAL_True;
}

