/*******************************************************************************
**+--------------------------------------------------------------------------+**
*
**|         	Copyright (c) Aztech System Ltd           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

#ifndef __SWITCH_CSL_ICPLUS_175C_H__
#define __SWITCH_CSL_ICPLUS_175C_H__


/*******************************************************************************
*
* cslICPlus175CStart - Does the primary initialization of the switch.
*
* Performs the initialization of the switch and switch specific SW data.
*
* INPUTS:
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/
Int32 cslICPlus175CStart (void);

/* According to operation type and port number we can retreive the PHY address */
/*******************************************************************************
*
* cslICPlus175CRegVal2OperStatus - extract the user operational status out of register value.
*
* Enable/Disable a selected port
*
* INPUTS:
*     [In]    OPER_TYPE oper 
*     [In]    UINT_16   data
* 
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslICPlus175CRegVal2OperStatus (OPER_TYPE_E oper, Uint16 data);

/*******************************************************************************
*
* cslICPlus175COperStatus2RegVal - constructs the register value to be written out of data and user request
*
* Enable/Disable a selected port
*
* INPUTS:
*     [In]    OPER_TYPE     oper 
*     [In]    Int32        operValue
*     [In]    UINT_16       data
* 
*
*
* RETURNS: 
*       The final register value
*
*
*/
Uint16 cslICPlus175COperStatus2RegVal (OPER_TYPE_E oper,Int32 operValue,Uint16 data);


/*******************************************************************************
*
* cslICPlus175CSwitchGetVersion - Get switch specific data
*
* 
*
* INPUTS:
*     [In]    PSW_VER_INFO pSwitchInfo
* 
*
*
* RETURNS: 
*       The final register value
*
*
*/

void cslICPlus175CSwitchGetVersion(PSW_VER_INFO pSwitchVerInfo);








/*******************************************************************************
*
* cslICPlus175CPortNum2RegAddr - returns the register address
*
* Retreive register address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/

Uint32 cslICPlus175CPortNum2RegAddr(OPER_TYPE_E type, Uint32 portNum);

/*******************************************************************************
*
* cslICPlus175CPortNum2PHYAddr - returns the PHY address
*
* Retreive PHY address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/


Uint32 cslICPlus175CPortNum2PHYAddr(OPER_TYPE_E type, Uint32 portNum);

/*******************************************************************************
*
* cslICPlus175CVlanDefaultSet - Defines the default VLAN
*
* Defines the default VLAN. The meaning can change from implementation to implementation.
*
* INPUTS:
*     [In]   Uint32         portNum
*     [In]   PCSL_SWITCH_VLAN_TABLE pVlnEntry
*
*
* RETURNS: 
*       Status
*
*
*/


Int32 cslICPlus175CVlanDefaultSet(Uint32 portNum,PCSL_SWITCH_VLAN_TABLE pVlnEntry);

/*******************************************************************************
*
* cslICPlus175CVlanUpdate - Updates the VLAN in VLAN table
*
* Writes the VLAN data to the VLAN table.
*
* INPUTS:
*     [In]   PCSL_SWITCH_VLAN_TABLE pVlnEntry
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslICPlus175CVlanUpdate(PCSL_SWITCH_VLAN_TABLE pVlnEntry);

/*******************************************************************************
*
* cslICPlus175CSpecialCapsSet - sets special capabilities configuration of the switch
*   
*
* INPUTS:
*     [In]   Uint32                 portNum
*     [In]   Uint32                 arg -the value of the specifice capability
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslICPlus175CSpecialCapsSet(Uint32 cmd, Uint32 arg);

#endif /* __SWITCH_CSL_ICPLUS_175C_H__*/


