#include "_tistdtypes.h"
#include "pal_defs.h"
#include "ddc_cpmacDrv.h"

#include "cpswhalcommon_stddef.h"
#include "cpswhalcommon_miimdio.h"

#include "ddc_switch.h"
#include "csl_switchPrvt.h"
#include "ddc_cpmacDrv.h"


#define REALTEK_8305SC_STATUS_SHIFT_MASK   11
#define REALTEK_8305SC_STATUS_MASK         0x1

#define REALTEK_8305SC_LINK_SHIFT_MASK     2
#define REALTEK_8305SC_LINK_MASK           0x1

#define REALTEK_8305SC_DUPLEX_SHIFT_MASK   8
#define REALTEK_8305SC_DUPLEX_MASK         0x1

#define REALTEK_8305SC_DUPLEX_SET_SHIFT_MASK   8 

#define REALTEK_8305SC_SPEED_SHIFT_MASK    13
#define REALTEK_8305SC_SPEED_MASK          0x1

#define REALTEK_8305SC_SPEED_SET_SHIFT_MASK   13  

#define REALTEK_8305SC_FC_SHIFT_MASK       10
#define REALTEK_8305SC_FC_MASK             0x1

#define REALTEK_8305SC_AUTO_ENABLE_SHIFT_MASK      12
#define REALTEK_8305SC_AUTO_ENABLE_MASK            0x1

#define REALTEK_8305SC_AUTO_RESTART_SHIFT_MASK     9   
#define REALTEK_8305SC_AUTO_RESTART_MASK           0x1


#define PHY_ADDR(portNum)       portNum - 1

#define INTERNAL_PORT_NUMBER 0x05

PUBLIC PHY_DEVICE * pSwitchInfo;


/*******************************************************************************
*
* cslRealtek8305scStart - Does the primary initialization of the switch.
*
* Performs the initialization of the switch and switch specific SW data.
*
* INPUTS:
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslRealtek8305scStart (void)
{
    Int32 retVal = -1;
/*    Uint32 regaddr,phyaddr,data;

    regaddr = 0x10;
    phyaddr = 0x4;

    data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
    data |= 0x2;        
    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);*/

    return retVal;
}


/*******************************************************************************
*
* cslRealtek8305scRegVal2OperStatus - extract the user operational status out of register value.
*
* This code contains switch specific information. The main purpose is converting switch specific info to 
* the neutral code. As a neutral data types we're using external interfaces.
*
* INPUTS:
*     [In]    OPER_TYPE oper 
*     [In]    Uint16   data
* 
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslRealtek8305scRegVal2OperStatus (OPER_TYPE_E oper, Uint16 data)
{
    Int32 retVal = -1;
    
    if (oper == OPER_STATUS)
    {
        retVal = (Int32)data & (REALTEK_8305SC_STATUS_MASK << REALTEK_8305SC_STATUS_SHIFT_MASK);
        return retVal;
    }

    else if (oper == OPER_LINK)
    {
        retVal = (Int32)data & (REALTEK_8305SC_LINK_MASK << REALTEK_8305SC_LINK_SHIFT_MASK);
        if(retVal == 0)
            return PORT_LINK_DOWN;
        return PORT_LINK_UP;
    }

    else if (oper == OPER_DUPLEX)
    {
        retVal = (Int32)data & (REALTEK_8305SC_DUPLEX_MASK << REALTEK_8305SC_DUPLEX_SHIFT_MASK);
        if(retVal == 0)
            return PORT_DUPLEX_HALF;
        return PORT_DUPLEX_FULL;
    }

    else if (oper == OPER_SPEED)
    {
        retVal = (Int32)data & (REALTEK_8305SC_SPEED_MASK << REALTEK_8305SC_SPEED_SHIFT_MASK);
        if(retVal == 0)
            return PORT_SPEED_10;
        return PORT_SPEED_100;
    }

    else if (oper == OPER_FC)
    {
//        retVal = (Int32)data & (REALTEK_8305SC_LINK_MASK << REALTEK_8305SC_LINK_SHIFT_MASK);
        retVal = (Int32)data & (REALTEK_8305SC_FC_MASK << REALTEK_8305SC_FC_SHIFT_MASK);
        if (retVal == 0)
            return PORT_FC_OFF;
        return PORT_FC_ON;
    }
    else if (oper == OPER_AUTO)
    {
        retVal = (Int32)data & (REALTEK_8305SC_AUTO_ENABLE_MASK << REALTEK_8305SC_AUTO_ENABLE_SHIFT_MASK);
        if (retVal == 0)
            return PORT_AUTO_NO;
        return PORT_AUTO_YES;
    }
    
    return retVal;
}

/*******************************************************************************
*
* cslRealtek8305scOperStatus2RegVal - constructs the register value to be written out of data and user request
*
* Enable/Disable a selected port
*
* INPUTS:
*     [In]    OPER_TYPE     oper 
*     [In]    Int32        operValue
*     [In]    Uint16       data
* 
*
*
* RETURNS: 
*       The final register value
*
*
*/

Uint16 cslRealtek8305scOperStatus2RegVal (OPER_TYPE_E oper,Int32 operValue,Uint16 data)
{
    Uint16 retVal = 0xffff;

    
    if (oper == OPER_STATUS)
    {
        retVal = (data & (~ ((Uint16)(REALTEK_8305SC_STATUS_MASK << REALTEK_8305SC_STATUS_SHIFT_MASK)))) | ((Uint16)operValue);
    }
    else if (oper == OPER_DUPLEX_SET)
    {
        retVal = data & (~((Uint16)(REALTEK_8305SC_DUPLEX_MASK << REALTEK_8305SC_DUPLEX_SET_SHIFT_MASK)));
        if(operValue == PORT_DUPLEX_FULL)
            retVal |= (REALTEK_8305SC_DUPLEX_MASK << REALTEK_8305SC_DUPLEX_SET_SHIFT_MASK);

        /* assert software reset, otherwise the changes will be ignored */
        retVal |= (Uint16)(REALTEK_8305SC_AUTO_RESTART_MASK << REALTEK_8305SC_AUTO_RESTART_SHIFT_MASK);  
    }
    else if (oper == OPER_SPEED_SET)
    {
        retVal = data & (~((Uint16)(REALTEK_8305SC_SPEED_MASK << REALTEK_8305SC_SPEED_SET_SHIFT_MASK)));
        if(operValue == PORT_SPEED_100)
            retVal |= (REALTEK_8305SC_SPEED_MASK << REALTEK_8305SC_SPEED_SET_SHIFT_MASK);

        /* assert software reset, otherwise the changes will be ignored */
        retVal |= (Uint16)(REALTEK_8305SC_AUTO_RESTART_MASK << REALTEK_8305SC_AUTO_RESTART_SHIFT_MASK); 
    }
    else if (oper == OPER_FC)
    {
        retVal = data & (~((Uint16)(REALTEK_8305SC_FC_MASK << REALTEK_8305SC_FC_SHIFT_MASK)));
        if(operValue == PORT_FC_ON)
            retVal |= (REALTEK_8305SC_DUPLEX_MASK << REALTEK_8305SC_DUPLEX_SHIFT_MASK);
    }
    else if (oper == OPER_AUTO)
    {
        retVal = data & (~((Uint16)(REALTEK_8305SC_AUTO_ENABLE_MASK << REALTEK_8305SC_AUTO_ENABLE_SHIFT_MASK)));
        if(operValue != PORT_AUTO_NO)
            retVal |= (Uint16)(REALTEK_8305SC_AUTO_ENABLE_MASK << REALTEK_8305SC_AUTO_ENABLE_SHIFT_MASK);
        
        /* assert software reset, otherwise the changes will be ignored */
        retVal |= (Uint16)(REALTEK_8305SC_AUTO_RESTART_MASK << REALTEK_8305SC_AUTO_RESTART_SHIFT_MASK); 
    }
    else if (oper == OPER_RESET_STAT)
    {
/*        retVal = (MARVELL_6063_STAT_STATSBUSY << MARVELL_6063_STAT_SHIFT_STATSBUSY) | 
           (MARVELL_6063_STAT_MODULE_RESET_PORT << MARVELL_6063_STAT_SHIFT_STATSOP) |
           (operValue << MARVELL_6063_STAT_SHIFT_STATSPR);*/
    }
    return retVal;
}

/*******************************************************************************
*
* cslRealtek8305scSwitchGetVersion - returns the switch specific information
*
* 
*
* INPUTS:
*     [Out]   PSW_VER_INFO pSwitchInfo
* 
*
*
* RETURNS: 
*       Status
*
*
*/

void cslRealtek8305scSwitchGetVersion(PSW_VER_INFO pSwitchVerInfo)
{
    Uint32 regaddr,phyaddr;

    regaddr = 0x3;
    phyaddr = 0x0;
    pSwitchVerInfo->is802_1_P_capable      = PAL_False; 
    pSwitchVerInfo->is802_1_Q_capable      = PAL_True;
    pSwitchVerInfo->maxNumberOfPorts       = 5;
    pSwitchVerInfo->maxNumberOfPriorities  = 0;
    pSwitchVerInfo->maxNumberOfVLANs       = 16;

    pSwitchVerInfo->Model = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
}

         
/*******************************************************************************
*
* cslRealtek8305scPortNum2RegAddr - returns the register address
*
* Retreive register address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/

Uint32 cslRealtek8305scPortNum2RegAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 regAddr = 0;

    switch (type) 
    { 
    case OPER_STATUS:
        regAddr = 0x0; 
        break; 
    case OPER_LINK: 
        regAddr = 0x1; 
        break; 
    case OPER_DUPLEX: 
        regAddr = 0x0; 
        break; 
    case OPER_SPEED: 
        regAddr = 0x0; 
        break; 
    case OPER_AUTO: 
        regAddr = 0x0; 
        break; 
    case OPER_FC: 
        regAddr = 0x4; 
        break; 
    case OPER_RESET_STAT: /*no use in realtek set regAddr 0x1F*/
        regAddr = 0x1F; 
        break; 
    default: 
    break; 
    } 

    return regAddr;
}

/*******************************************************************************
*
* cslRealtek8305scPortNum2PHYAddr - returns the PHY address
*
* Retreive PHY address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/


Uint32 cslRealtek8305scPortNum2PHYAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 phyAddr = 0;
    
    switch (type) 
    { 
    case OPER_STATUS: 
        phyAddr = PHY_ADDR(portNum);
        break; 
    case OPER_LINK: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_DUPLEX: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_DUPLEX_SET: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_SPEED: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_SPEED_SET: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_AUTO: 
        phyAddr = PHY_ADDR(portNum); 
        break; 
    case OPER_FC: 
        phyAddr = PHY_ADDR(portNum);
        break; 
    case OPER_VLANTUNNEL: 
        phyAddr = 0x1F;
        break; 
    case OPER_RESET_STAT: /*no use in realtek set port numboer to 0xF*/
        phyAddr = 0x1F; 
        break; 
    default: 
    break; 
    }

    return phyAddr;
}
/**************************************************** VLAN submodule **************************************/

/*******************************************************************************
*
* cslRealtek6063VlanDefaultSet - Defines the default VLAN
*
* Defines the default VLAN. The meaning can change from implementation to implementation.
*
* INPUTS:
*     [In]   Uint32         portNum
*     [In]   PCSL_SWITCH_VLAN_TABLE pVlnEntry
*
*
* RETURNS: 
*       Status
*
*
*/


Int32 cslRealtek8305scVlanDefaultSet(Uint32 portNum,PCSL_SWITCH_VLAN_TABLE pVlnEntry)
{
    Uint32 regaddr,phyaddr,data;

    regaddr = 25;
    phyaddr = PHY_ADDR(portNum);
    data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
    data = (data & 0xF000) | (pVlnEntry->vlanId & 0xFFF);
    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
    return PAL_True;
}

/*******************************************************************************
*
* cslRealtek8305scSpecialCapsSet - sets special capabilities configuration of the switch
*   
*
* INPUTS:
*     [In]   Uint32                 portNum
*     [In]   Uint32                 arg -the value of the specifice capability
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslRealtek8305scSpecialCapsSet(Uint32 cmd, Uint32 arg)
{
    Uint32 regaddr,phyaddr,data,port;                    
    Uint32 indata;
    INGRESS_VLN_MODE mode;

    switch ((OPER_TYPE_E)cmd)
    {
/* all the ports are ingress checkin/ no ingress checking */
    case OPER_VLAN_TABLE_SET:

        port = (arg & 0xF000)>>12;

        if (port > 5)
            return -1;

        arg &= 0x1F;
        phyaddr = PHY_ADDR(port);

        regaddr = 24;
        data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
        data = ( (data & 0xDE0) | arg) |( phyaddr << 12);   
        _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data); 
        regaddr = 25;
        data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
        data = (data & 0xF000) | (port & 0xFFF);
        _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data); 
        break;

    case OPER_INGRESS:
    {
        INGRESS_VLN_MODE mode = (INGRESS_VLN_MODE)arg;
        phyaddr = 0;
        switch(mode){
        default:
        case INGRESS_VLN_MODE_8021Q_DISABLE:
            regaddr = 16;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0x79FF) | 0x700;
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);        
            regaddr = 18;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0xFEFF) | 0x100;
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
            data = 0x3;        
        break;
        case INGRESS_VLN_MODE_8021Q_CHECK:
            regaddr = 16;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0x78FF) | 0x8300;
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);        
            regaddr = 18;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0xFEFF);
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
            data = 0x0;
        break;
        case INGRESS_VLN_MODE_8021Q_SECURE:
            regaddr = 16;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0x79FF) | 0x8100;
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);        
            regaddr = 18;
            data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data = (data & 0xFEFF);
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
            data = 0x2;
        break;
        }
        if (data != 3){
            regaddr = 22;
            phyaddr = 4;
            indata = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
            data |= (indata & 0xFFFC);
            _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
            data = 0x1;
            for (phyaddr = 0; phyaddr < 4; phyaddr++){
                indata = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
                data |= (indata & 0xFFFC);
                _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);
            }
        }
    }            
        break;

    case OPER_INGRESS_OFF:
        phyaddr = 0;
        regaddr = 16;
        data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
        data = (data & 0xF9FF) | 0x700;
        _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);        
        regaddr = 18;
        data = _cpswHalCommonMiiMdioUserAccessRead(pSwitchInfo,regaddr, phyaddr);
        data = (data & 0xFEFF) | 0x100;
        _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);        
       break;
    case OPER_PORT_RECOGNITION_ON:   
        break;

    case OPER_TRANSMIT_ON_FIXED_PORT:
        break;

    case OPER_VLANTUNNEL:
    default:
            break;           

    }

    return PAL_True;
}


#define ATU_PURGE_OPERATION_VALUE 0xB000
/*******************************************************************************
*
* cslRealtek8305scPurgeAtuEntry - Purge an entry from the address translation Unit table.
*
*
*
* INPUTS:
*     [In]   char MacAddress[6]
*
*
* RETURNS: 
*       Status
*
*
*/

Int32 cslRealtek8305scPurgeAtuEntry(char MacAddress[6])
{
    Uint32 regaddr,phyaddr;
    Uint32 data = 0;
    unsigned short data1=0;

    phyaddr = 0x4;

   // ATU MAC ADDRESS Register Bytes 0 & 1 
    data1 = ((MacAddress[0] << 8) | MacAddress[1] );
    regaddr = 0x14;
    
    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,(Uint32)data1);

   // ATU MAC ADDRESS Register Bytes 2 & 3 

    data1 = ((MacAddress[2] << 8) | MacAddress[3] );     
    regaddr = 0x13;
    
    
    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,(Uint32)data1);
   
    // ATU MAC ADDRESS Register Bytes 4 & 5 
    data1 = ((MacAddress[4] << 8) | MacAddress[5] );

    regaddr = 0x12;
        
    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,(Uint32)data1);

    // we're ready to write the data to ATU, perform it 
    // ATU Data Register - Entry State must be zero for purge
/*    regaddr = 0x0C;
        
    data = 0x00;

    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);*/

    // ATU Operation Register 
    regaddr = 0x11;
    // Set Opcode to Purge and set on the ATUBusy bit 
    data = 0x2F;

    _cpswHalCommonMiiMdioUserAccessWrite(pSwitchInfo,regaddr, phyaddr,data);

    return PAL_True;
}

