/******************************************************************************   
 * FILE PURPOSE:     - LED kenel Header                                           
 ******************************************************************************   
 * FILE NAME:     led_hal_drv.h                                                     
 *                                                                                
 * DESCRIPTION:   Header file defining HAL types and functions.
 *                                                                                
 *                                                                                
 * (C) Copyright 2004, Texas Instruments, Inc                                     
 *******************************************************************************/ 
#ifndef __LED_KERNEL__
#define __LED_KERNEL__

#define LED_MAX_GPIO_INPUTS     5
#define LED_MAX_BITS			2   /* Max bits (GPIO pins) for a LED (max=2)*/

/* The following structure is used for kernel modules which supply non-GPIO LEDs */
struct led_funcs
{
  int param;
  void ( *led_init )( int param );
  void ( *led_on )  ( int param );
  void ( *led_off ) ( int param );
};

struct led_object
{
  struct led_funcs  *led_funcs;           /* LED functions for non-gpio LED (NULL=GPIO)*/

  unsigned int num_of_gpios;              /* How many GPIO pins used by the LED (def=1)*/
  unsigned int led_gpios[ LED_MAX_BITS ]; /* GPIO numbers */
};

struct gpio_input_state
{
  int gpio_num;
  int gpio_state;
  int invert;
  int state_change;
};

/******* Kernel APIs *********/

/* Initializes the hal_dev structure and timers */
int led_hal_init(unsigned int gpio_off_value, void *drv_data, unsigned int args );

/* Releases the hal_dev structure and timers */
int led_hal_exit(void);

/* Registers a LED */
int led_hal_user_register(const struct led_gpio_object *led);

/* Registers a LED, returns LED handle or negative value on error */
int led_hal_register(const struct led_object *led);

/* Deregisters a registered LED. Returns 0 for OK or negative value on error */
int led_hal_deregister(int led_handle);

/* Perform an action on a LED. Returns 0 for OK or negative value on error */
int led_hal_action(const struct led_action *led_action);

/* Registers an input GPIO, returns a handle or negative value on error */
int led_hal_input_register(const struct input_gpio_object *gpio_obj);

/* Deregisters a registered input GPIO. Returns 0 for OK or negative value on error */
int led_hal_input_deregister(int gpio_handle);

/* Retrieves the current status of an Input GPIO */
int led_hal_get_input_status(int index, struct gpio_input_state *gpio);

/* HAL control function to set input_timeout value */
int led_hal_control(int input_timeout);

#endif
