/*
 * This is the 1999 rewrite of IP Firewalling, aiming for kernel 2.3.x.
 *
 * Copyright (C) 1999 Paul `Rusty' Russell & Michael J. Neuling
 *
 * Extended to all five netfilter hooks by Brad Chapman & Harald Welte
 */
//-------------------------------------------------------------------------------------
// Copyright 2005, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
// 
// 1. NSP QoS Framework 
// 2. Instrumentation
//
// THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
// "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS 
// OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY 
// PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR 
// DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, 
// COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. 
// See The GNU General Public License for more details.
//
// These changes are covered under version 2 of the GNU General Public License, 
// dated June 1991.
//-------------------------------------------------------------------------------------

#include <linux/config.h>
#include <linux/module.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netdevice.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <net/route.h>
#include <linux/ip.h>

#include <linux/psp_trace.h>

#define MANGLE_VALID_HOOKS ((1 << NF_IP_PRE_ROUTING) | \
			    (1 << NF_IP_LOCAL_IN) | \
			    (1 << NF_IP_FORWARD) | \
			    (1 << NF_IP_LOCAL_OUT) | \
			    (1 << NF_IP_POST_ROUTING))

/* This rwlock protects the main hash table, protocol/helper/expected
   registrations, conntrack timers*/
#define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ip_conntrack_lock)
#define ASSERT_WRITE_LOCK(x) MUST_BE_WRITE_LOCKED(&ip_conntrack_lock)

#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>

/* Standard entry. */
struct ipt_standard
{
	struct ipt_entry entry;
	struct ipt_standard_target target;
};

struct ipt_error_target
{
	struct ipt_entry_target target;
	char errorname[IPT_FUNCTION_MAXNAMELEN];
};

struct ipt_error
{
	struct ipt_entry entry;
	struct ipt_error_target target;
};

/* Ouch - five different hooks? Maybe this should be a config option..... -- BC */
static struct
{
	struct ipt_replace repl;
	struct ipt_standard entries[5];
	struct ipt_error term;
} initial_table __initdata
= { { "mangle", MANGLE_VALID_HOOKS, 6,
      sizeof(struct ipt_standard) * 5 + sizeof(struct ipt_error),
      { [NF_IP_PRE_ROUTING] 	0,
	[NF_IP_LOCAL_IN] 	sizeof(struct ipt_standard),
	[NF_IP_FORWARD] 	sizeof(struct ipt_standard) * 2,
	[NF_IP_LOCAL_OUT] 	sizeof(struct ipt_standard) * 3,
	[NF_IP_POST_ROUTING] 	sizeof(struct ipt_standard) * 4 },
      { [NF_IP_PRE_ROUTING] 	0,
	[NF_IP_LOCAL_IN] 	sizeof(struct ipt_standard),
	[NF_IP_FORWARD] 	sizeof(struct ipt_standard) * 2,
	[NF_IP_LOCAL_OUT] 	sizeof(struct ipt_standard) * 3,
	[NF_IP_POST_ROUTING]	sizeof(struct ipt_standard) * 4 },
      0, NULL, { } },
    {
	    /* PRE_ROUTING */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* LOCAL_IN */
 	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* FORWARD */
 	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* LOCAL_OUT */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* POST_ROUTING */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
    },
    /* ERROR */
    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
	0,
	sizeof(struct ipt_entry),
	sizeof(struct ipt_error),
	0, { 0, 0 }, { } },
      { { { { IPT_ALIGN(sizeof(struct ipt_error_target)), IPT_ERROR_TARGET } },
	  { } },
	"ERROR"
      }
    }
};

static struct ipt_table packet_mangler
= { { NULL, NULL }, "mangle", &initial_table.repl,
    MANGLE_VALID_HOOKS, RW_LOCK_UNLOCKED, NULL, THIS_MODULE };

/**************************************************************************
 * FUNCTION NAME : ti_nsp_qos_pr_hook
 **************************************************************************
 * DESCRIPTION   :
 *  This function handles the QoS and Policy Routing code. It manipulates the
 *  "nfmark" field.
 ***************************************************************************/
static void ti_nsp_qos_pr_hook (struct sk_buff **pskb, unsigned int hook)
{
	struct ip_conntrack*    ct;
	enum ip_conntrack_info  ctinfo;

    /* Get the conntrack information... */
	ct = ip_conntrack_get(*pskb, &ctinfo);

    /* The POLICY Routing and QoS Framework work need to be done only in the PRE Routing hook.  */
    if ((ct != NULL) && ((hook == NF_IP_PRE_ROUTING) || (hook == NF_IP_LOCAL_OUT)))
    {
        /* Check if the Qs bit was set */
        if ((*pskb)->nfmark & 0x8000)
        {
            int q  = ((*pskb)->nfmark & 0x0007);

            /* YES. The Qs bit was set. Validate the Class of services and appropriately mark the priority 
             * of the connection. */
            if (q >= TI_NSP_QOS_COS1 && q <= TI_NSP_QOS_COS6)
            {
                /* Mark the priority of the connection tracking entry. */
                ct->conntrack_priority = q;
            }
            else
            {
                /* TODO: This message should be removed after the configuration manager has been validated. */
                printk ("DEBUG: MANGLE QoS/PR Encoding violated; defaulting to CoS6\n");
                ct->conntrack_priority = TI_NSP_QOS_COS6;
            }
        }

        /* Syncing up the connection tracking priority with the priority in the packet. 
         * The following are cases where the packet inside the packet is NOT correct:-
         *    (a) When the packet is a fragment. There is not sufficient info to make the decision
         *    (b) For any packet that relies on the payload database to determine the priority.
         * For all other modes the priority inside the packet is always correct. */
        if ((*pskb)->qos_pkt_info.ti_qos_flags & (TI_QOS_PACKET_LIE | TI_QOS_PACKET_FRAGMENTED))
        {
            (*pskb)->qos_pkt_info.ti_service_class = ct->conntrack_priority;
        }
        else
        {
            ct->conntrack_priority = (*pskb)->qos_pkt_info.ti_service_class;
        }

        /* Check if the Ps bit was set */
        if ((*pskb)->nfmark & 0x4000)
        {
            /* YES. The Ps bit was set. Mark the packet with the "p" bits. */
            (*pskb)->nfmark = ((*pskb)->nfmark & 0x3FF8) >> 3;
        }
    }

    /* The work is complete. */
    return;
}

/* The work comes in here from netfilter.c. */
static unsigned int
ipt_route_hook(unsigned int hook,
	 struct sk_buff **pskb,
	 const struct net_device *in,
	 const struct net_device *out,
	 int (*okfn)(struct sk_buff *))
{
    unsigned int            retVal;

    /* Trace Point: Start. */
    psp_trace(IP_MANGLE_ENTER);

    /* Look at the Packet Mangler Database. */
    retVal = ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);

    /* Call the QoS and PR Hook. */
    ti_nsp_qos_pr_hook(pskb, hook);

    /* Trace Point: Stop. */
    psp_trace(IP_MANGLE_EXIT);
    return retVal;
}

static unsigned int
ipt_local_hook(unsigned int hook,
		   struct sk_buff **pskb,
		   const struct net_device *in,
		   const struct net_device *out,
		   int (*okfn)(struct sk_buff *))
{
	unsigned int ret;
	u_int8_t tos;
	u_int32_t saddr, daddr;
	unsigned long nfmark;

	/* root is playing with raw sockets. */
	if ((*pskb)->len < sizeof(struct iphdr)
	    || (*pskb)->nh.iph->ihl * 4 < sizeof(struct iphdr)) {
		if (net_ratelimit())
			printk("ipt_hook: happy cracking.\n");
		return NF_ACCEPT;
	}

	/* Save things which could affect route */
	nfmark = (*pskb)->nfmark;
	saddr = (*pskb)->nh.iph->saddr;
	daddr = (*pskb)->nh.iph->daddr;
	tos = (*pskb)->nh.iph->tos;

	ret = ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);

    /* Call the QoS and PR Hook. */
    ti_nsp_qos_pr_hook(pskb, hook);

	/* Reroute for ANY change. */
	if (ret != NF_DROP && ret != NF_STOLEN && ret != NF_QUEUE
	    && ((*pskb)->nh.iph->saddr != saddr
		|| (*pskb)->nh.iph->daddr != daddr
		|| (*pskb)->nfmark != nfmark
		|| (*pskb)->nh.iph->tos != tos))
		return ip_route_me_harder(pskb) == 0 ? ret : NF_DROP;

	return ret;
}

static struct nf_hook_ops ipt_ops[]
= { { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_PRE_ROUTING, 
	NF_IP_PRI_MANGLE },
    { { NULL, NULL }, ipt_local_hook, PF_INET, NF_IP_LOCAL_IN,
	NF_IP_PRI_MANGLE },
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_FORWARD,
	NF_IP_PRI_MANGLE },
    { { NULL, NULL }, ipt_local_hook, PF_INET, NF_IP_LOCAL_OUT,
	NF_IP_PRI_MANGLE },
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_POST_ROUTING,
	NF_IP_PRI_MANGLE }
};

static int __init init(void)
{
	int ret;

	/* Register table */
	ret = ipt_register_table(&packet_mangler);
	if (ret < 0)
		return ret;

	/* Register hooks: PRE-ROUTING */
	ret = nf_register_hook(&ipt_ops[0]);
	if (ret < 0)
		goto cleanup_table;

	ret = nf_register_hook(&ipt_ops[1]);
	if (ret < 0)
		goto cleanup_hook0;

	ret = nf_register_hook(&ipt_ops[2]);
	if (ret < 0)
		goto cleanup_hook1;

	ret = nf_register_hook(&ipt_ops[3]);
	if (ret < 0)
		goto cleanup_hook2;

	ret = nf_register_hook(&ipt_ops[4]);
	if (ret < 0)
		goto cleanup_hook3;

	return ret;

 cleanup_hook3:
        nf_unregister_hook(&ipt_ops[3]);
 cleanup_hook2:
        nf_unregister_hook(&ipt_ops[2]);
 cleanup_hook1:
	nf_unregister_hook(&ipt_ops[1]);
 cleanup_hook0:
	nf_unregister_hook(&ipt_ops[0]);
 cleanup_table:
	ipt_unregister_table(&packet_mangler);

	return ret;
}

static void __exit fini(void)
{
	unsigned int i;

	for (i = 0; i < sizeof(ipt_ops)/sizeof(struct nf_hook_ops); i++)
		nf_unregister_hook(&ipt_ops[i]);

	ipt_unregister_table(&packet_mangler);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");


