/* text2ip.c - Converts a text string to an IP address. */

/* Written 1998 by Werner Almesberger, EPFL ICA */

//-------------------------------------------------------------------------------------
// Copyright 2005, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
// 
// 1. NSP Policy Routing Framework
//
// THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
// "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS 
// OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY 
// PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR 
// DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, 
// COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. 
//------------------------------------------------------------------------------------- 

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/pr.h>

#include "atm.h"
#include "atmd.h"


static void complain(const char *component,const char *item,const char *msg)
{
    if (!component) fprintf(stderr,"%s: %s\n",item,msg);
    else diag(component,DIAG_ERROR,"%s: %s",item,msg);
}


uint32_t text2ip(const char *text,const char *component,int flags)
{
    struct hostent *hostent;
    uint32_t ip;
    unsigned int pr_mark = LIBATM_PR_MARK;

    if (strspn(text,"0123456789.") == strlen(text)) {
	ip = inet_addr(text);
	if (ip != INADDR_NONE) return ip;
	if (flags & T2I_ERROR) complain(component,text,"invalid address");
	return INADDR_NONE;
    }
    if (!(flags & T2I_NAME)) {
	if (flags & T2I_ERROR)
	    complain(component,text,"numeric IP address expected");
	return INADDR_NONE;
    }

    /* Adding an option which allows to pass a mark value for a particular
     * socket. */
#if 0
    hostent = gethostbyname(text);
#else
    hostent = ti_gethostbyname(text, pr_mark);
#endif
    if (!hostent) {
	if (flags & T2I_ERROR) complain(component,text,"no such host");
	return INADDR_NONE;
    }
    if (hostent->h_addrtype != AF_INET) {
	if (flags & T2I_ERROR)
	    complain(component,text,"unknown address family");
	return INADDR_NONE;
    }
    memcpy(&ip,hostent->h_addr,hostent->h_length);
    return ip;
}
